/*
 * @(#) VehicleQuery.java
 *
 * This code is part of the JNavigator project.
 * Copyright (c) 2011  Clemens Krainer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
package at.uni_salzburg.cs.ckgroup.cscpp.engine.json;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.json.simple.JSONArray;
import org.json.simple.JSONValue;

import at.uni_salzburg.cs.ckgroup.cscpp.engine.parser.IAction;
import at.uni_salzburg.cs.ckgroup.cscpp.engine.parser.Position;
import at.uni_salzburg.cs.ckgroup.cscpp.engine.vehicle.IVirtualVehicle;
import at.uni_salzburg.cs.ckgroup.cscpp.utils.IQuery;
import at.uni_salzburg.cs.ckgroup.cscpp.utils.IServletConfig;

public class VehicleQuery implements IQuery {

	private static final String PROP_VEHICLE_LOCAL_NAME = "name";
	private static final String PROP_VEHICLE_STATE = "state";
	private static final String PROP_VEHICLE_LATITUDE = "latitude";
	private static final String PROP_VEHICLE_LONGITUDE = "longitude";
	private static final String PROP_VEHICLE_ALTITUDE = "altitude";
	private static final String PROP_VEHICLE_TOLERANCE = "tolerance";
	private static final String PROP_VEHICLE_ACTIONS = "actions";
	
	private Map<String, IVirtualVehicle> vehicleMap;
	
	@SuppressWarnings("serial")
	private Map<String,String> actionsMap = new HashMap<String, String>() {{
		put("AIRPRESSURE", "airPressure");
		put("ALTITUDE", "altitude");
		put("COURSE", "course");
		put("PICTURE", "photo");
		put("RANDOM", "random");
		put("SONAR", "sonar");
		put("SPEED", "speed");
		put("TEMPERATURE", "temperature");
	}};

	public void setVehicleMap(Map<String, IVirtualVehicle> vehicleMap) {
		this.vehicleMap = vehicleMap;
	}
	
	public String execute(IServletConfig config, String[] parameters) {

		Map<String, Object> obj=new LinkedHashMap<String, Object>();
		
		for (IVirtualVehicle vehicle : vehicleMap.values()) {
			if (vehicle.isFrozen()) {
				continue;
			}
			
			Map<String, Object> props = new LinkedHashMap<String, Object>();
			for (Entry<Object, Object> e : vehicle.getProperties().entrySet()) {
				props.put((String)e.getKey(), e.getValue());
			}
			
			props.put(PROP_VEHICLE_LOCAL_NAME, vehicle.getWorkDir().getName());
			
			if(vehicle.isProgramCorrupted()) {
				props.put(PROP_VEHICLE_STATE, "corrupt");
			} else if(vehicle.isCompleted()) {
				props.put(PROP_VEHICLE_STATE, "completed");
			} else if (vehicle.isActive()) {
				props.put(PROP_VEHICLE_STATE, "active");
			} else {
				props.put(PROP_VEHICLE_STATE, "suspended");
			}

			if (vehicle.getCurrentCommand() != null) {
				Position p = vehicle.getCurrentCommand().get_position();
				props.put(PROP_VEHICLE_LATITUDE, Double.valueOf(p.getPt().getLatitude()));
				props.put(PROP_VEHICLE_LONGITUDE, Double.valueOf(p.getPt().getLongitude()));
				props.put(PROP_VEHICLE_ALTITUDE, Double.valueOf(p.getPt().getAltitude()));
				props.put(PROP_VEHICLE_TOLERANCE, Double.valueOf(p.getTolerance()));
				
				List<IAction> al = vehicle.getCurrentCommand().get_actions();
//				StringBuilder b = new StringBuilder();
//				boolean first = true;
//				for (IAction action : al) {
//					if (action.getTimestamp() <= 0) {
//						continue;
//					}
//					if (!first) {
//						b.append(",");
//					}
//					b.append(actionsMap.get(action.toString().toUpperCase()));
//					first = false;
//				}
//				props.put(PROP_VEHICLE_ACTIONS, b.toString());
				JSONArray openActions = new JSONArray();
				for (IAction action : al) {
					if (!action.isComplete()) {
						openActions.add(actionsMap.get(action.toString().toUpperCase()));	
					}
				}
				props.put(PROP_VEHICLE_ACTIONS, openActions);
			}
			obj.put(vehicle.getWorkDir().getName(), props);
		}
		
		return JSONValue.toJSONString(obj);
	}

}
